<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

require_once JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php';

class SmartsubModelPayment extends JModelAdmin
{
	protected $text_prefix = 'COM_SMARTSUB_PAYMENT';

	protected function canDelete($record)
	{
		$user = JFactory::getUser();
		return $user->authorise('core.payment.delete', 'com_smartsub');
	}

	// check if the user can edit payment status
	protected function canEditState($record)
	{
		$user = JFactory::getUser();
		
		if($user->authorise('core.payment.edit.state', 'com_smartsub'))
		{
			// for online payments we allow state change only for users
			// who have payment.edit permissions also
			if(empty($record->offline))
			{
				return $user->authorise('core.payment.edit', 'com_smartsub');
			}
			return true;
		}
		// not authorised to edit state
		return false;
	}

	public function getTable($type = 'Payment', $prefix = 'SmartsubTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_smartsub.payment', 'payment', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}

		if(empty($data))
		{
			// when the form is open for edit, $data parameter is empty
			// we can still get data from the form object, so we can use in
			// in canEditState(), etc. for different types of payments
			// (offline/online)...
			$data = $form->getData()->toObject();
		}
		
		// Modify the form based on access controls.
		if (!$this->canEditState((object) $data))
		{
			// Disable fields for display.
			$form->setFieldAttribute('status', 'disabled', 'true');
			
			// Disable fields while saving.
			$form->setFieldAttribute('status', 'filter', 'unset');
		}
		
		$user = JFactory::getUser();
		if(!$user->authorise('core.payment.edit', 'com_smartsub'))
		{
			// Disable fields for edit.
			$form->setFieldAttribute('account_id', 'disabled', 'true');
			$form->setFieldAttribute('expires', 'disabled', 'true');
			$form->setFieldAttribute('amount', 'disabled', 'true');
			$form->setFieldAttribute('currencyID', 'disabled', 'true');
			$form->setFieldAttribute('transactionID', 'disabled', 'true');
			
			// Disable fields while saving.
			
			$form->setFieldAttribute('account_id', 'filter', 'unset');
			$form->setFieldAttribute('expires', 'filter', 'unset');
			$form->setFieldAttribute('amount', 'filter', 'unset');
			$form->setFieldAttribute('currencyID', 'filter', 'unset');
			$form->setFieldAttribute('transactionID', 'filter', 'unset');
			
		}
		
		// payment is available for edit only, so we can be sure that we have
		// account_id on the form
		$account_id = $form->getValue('account_id');
		$accounts_table = JTable::getInstance('Account', 'SmartsubTable');
		if($accounts_table->load($account_id) && $accounts_table->amount == 0)
		{
			// if we are treating with a free account, reset some values
			$form->setValue('expires', '', '');
			$form->setValue('amount', '', '0');
			$form->setFieldAttribute('expires', 'readonly', 'true');
			$form->setFieldAttribute('amount', 'readonly', 'true');
			$form->setFieldAttribute('currencyID', 'readonly', 'true');
		}
		
		return $form;
	}

	protected function loadFormData()
	{
		// Look at post for form data (for onchange="this.form.submit();")
		$data = JRequest::getVar('jform', array(), 'post', 'array');
		if(empty($data))
		{
			// Check the session for previously entered form data.
			$data = JFactory::getApplication()->getUserState('com_smartsub.edit.payment.data', array());
		}
		if (empty($data))
		{
			$data = $this->getItem();
		}

		$this->preprocessData('com_smartsub.payment', $data);

		return $data;
	}

	public function getItem($pk = null)
	{
		$item = parent::getItem($pk);
		
		$payment_systems = SmartsubHelper::getPaymentMethodNames();
		if(isset($payment_systems[$item->system]))
		{
			$item->payment_method = $payment_systems[$item->system];
		}

		return $item;
	}

	protected function prepareTable($table)
	{
		return parent::prepareTable($table);
	}

	public function save($data)
	{
		if(empty($data['id']))
		{
			$this->setError(JText::_('COM_SMARTSUB_PAYMENT_ERROR_CANNOT_CREATE'));
			return false;
		}
		
		$app = JFactory::getApplication();
		
		// get old status
		$table = $this->getTable();
		if(!$table->load($data['id']))
		{
			$this->setError(JText::_('JLIB_DATABASE_ERROR_EMPTY_ROW_RETURNED'));
			return false;
		}
		
		// special case for free accounts
		$accounts_table = JTable::getInstance('Account', 'SmartsubTable');
		// $data['account_id'] will be not set if the user has no permissions to edit payments
		// just ignore it in these cases
		if(isset($data['account_id']) && $accounts_table->load($data['account_id']) && $accounts_table->amount == 0)
		{
			// reset some properties if current account is free
			$data['expires'] = $this->getDbo()->getNullDate();
			$data['amount'] = 0;
			$data['system'] = 'none';
			if($data['status'] != 'paid' && $data['status'] != 'archived')
			{
				// Do not allow statuses except 'paid' and 'archive'
				$data['status'] = 'archived';
			}
			$data['offline'] = 0;
		}
		
		$result = parent::save($data);
		
		$id = $this->getState('payment.id');
		
		// if paid status is set always call setPaid(), as it is responsable
		// for updating user profile
		// $data['status'] will be not set if the user has no permissions to edit payments status
		// just ignore it in these cases
		if(isset($data['status']) && $data['status'] == 'paid')
		{
			try 
			{
				$this->setPaid($id, $data['status']);
			}
			catch(RuntimeException $e)
			{
				$this->setError($e->getMessage());
				return false;
			}
		}

		return true;
	}
	
	/**
	 * Set payment status. If status is 'paid', update related user profile with account and
	 * expiration date from pr_payments table row. 
	 * 
	 * @param array of int $pks - payment ids
	 * @param string $status - status to set
	 * @throws Exception
	 * @return integer - number of rows processed
	 */
	public function setPaid($pks, $status)
	{
		$app = JFactory::getApplication();
		$db = $this->getDbo();
		$task = $app->input->get('task');
		
		$user = JFactory::getUser();
		
		// Access checks. 'dotransaction' controller task has already checked that the payment can be set as paid
		if ($task != 'dotransaction' && !$user->authorise('core.payment.edit.state', 'com_smartsub'))
		{
			throw new Exception(JText::_('JLIB_APPLICATION_ERROR_EDITSTATE_NOT_PERMITTED'));
		}

		$table = $this->getTable();
		
		$count = 0;
		
		if(!is_array($pks))
		{
			$pks = array($pks);
		}
		
		// Sort $pks, so that the latest payment will take effect on user profile
		if($status == 'paid')
		{
			$query = $db->getQuery(true);
			$query->select('p.id');
			$query->from('#__ss_payments AS p');
			$query->where('p.id IN (' . implode(',', $pks) . ')');
			$query->order('p.date ASC');
			$db->setQuery($query);
			
			$pks = $db->loadColumn();
			if(empty($pks))
			{
				throw new Exception(JText::_('COM_SMARTSUB_NO_PAYMENT_SELECTED'));
			}
		}
		
		$is_list_operation = $task == 'setpaid' || $task == 'setpending';
		
		foreach($pks as $pk)
		{
			if(!$table->load($pk))
			{
				throw new Exception(JText::_('JLIB_DATABASE_ERROR_EMPTY_ROW_RETURNED'));
			}
			
			if($table->offline == 0 && $is_list_operation)
			{
				// skip online payments if not set explicitly on edit form or called
				// from payment controller
				continue;	
			}
			
			// update user profile if new status is 'paid' and the status has changed
			// Also update profile for paid and if called from payment save form or
			// payment controller (for online payments)
			if($status == 'paid' && ($table->status != $status || !$is_list_operation))
			{
				$account_user = JUser::getInstance($table->user_id);
				if($account_user->get('id', 0) == 0)
				{
					throw new Exception(JText::_('JLIB_DATABASE_ERROR_EMPTY_ROW_RETURNED'));
				}
				
				$profile = JUserHelper::getProfile($table->user_id);
				$profile->smartsub['account'] = $table->account_id;
				$account_user->set('smartsub', $profile->smartsub);
				
				if(!$account_user->save())
				{
					throw new Exception(JText::_('COM_SMARTSUB_USER_PROFILE_SAVE_FAILED'));
				}
				
				// Check in the profile.
				JModelLegacy::addIncludePath(JPATH_ROOT . '/components/com_users/models', 'UsersModel');
				$model	= JModelLegacy::getInstance('Profile', 'UsersModel', array());
				$model->checkin($table->user_id);
			}
			
			if($table->status != $status)
			{
				$count++;	
			}
			
			// for statuses other than 'paid' we do not have to update user profile,
			// just update the row in pr_payments table and onUserLogin() plugin
			// method will refuse the user in logging in
			$table->status = $status;
			
			// Force row checkin
			$table->checked_out = 0;
			$table->checked_out_time = $db->getNullDate();
			
			if(!$table->store())
			{
				throw new Exception($table->getError());
			}
		}
		
		// Clean the cache.
		$this->cleanCache();

		return $count;
	}
}
